/*
 ** Data structure and start of workers
 */

  use Math;
  use Worker;
  use NodeModule;
  use BlockDist;

  config const numWorkersPerLoc: int = 3;
  config const k: int = 10;   // task block size
  config const privateSize: int = 2 * k;
  config const publicSize: int = 10000 * k;  

  const numWorkers: int = numWorkersPerLoc * numLocales;
  const thrDomain = {0 .. #numWorkers};
  const thrDomainBlocked = thrDomain dmapped Block(thrDomain);

  const sqSize = publicSize + privateSize;
  const sqDomain = {0 .. #sqSize};

  var poolComm: [thrDomainBlocked] Worker;

  // Initialize pool including references to workers
  proc initPool() {
    coforall loc in Locales do on loc {
      // Parallelization does not seem worthwhile
      for localID in 0..#numWorkersPerLoc {
        const globalID = numWorkersPerLoc * here.id + localID;
        poolComm[globalID] = new Worker(globalID);
      }
    }
  }

  // Run workers and count tree nodes
  proc compute(): uint {
    var nNodes$ : sync uint = 0;
    coforall loc in Locales do on loc {
      var localeNNodes$: sync uint = 0;
      coforall localID in 0..#numWorkersPerLoc {
        const globalID = numWorkersPerLoc * here.id + localID;
        localeNNodes$ += poolComm[globalID].run();
      }
      nNodes$ += localeNNodes$;  
    }
    return nNodes$;
  }

