/*
** Main function and setting of tree parameters
**
** Part of this file was adapted from 
** https://chapel.svn.sourceforge.net/svnroot/chapel/trunk/test/studies/uts/
*/

  use Math;
  use Time;
  use sha1_rng;
  use Pool;
  use NodeModule;

  enum NodeDistrib { Binomial, Geometric, Hybrid, Constant };
  enum GeoDistrib  { GeoFixed, GeoLinear, GeoPoly, GeoCyclic };

  config const SEED:         int = 16;    // Seed for the hash of the root node
  config const MAX_CHILDREN: int = 10;    // Max. Children a node may have
  config const B_0:          int = 4;     // Branching Factor
  config const MAX_HEIGHT:   int = 6;     // Height limit
  config const nonLeafProb:  real = 0.10; // Probability of a non-leaf (binomial)
  config const nonLeafBF:    int = 4;     // Non-leaf branching factor (binomial)
  config const shiftHeight:  real = 0.5;  // Height at which hybrid trees go Geo=>Bin

  config const distrib: NodeDistrib = NodeDistrib.Geometric;
  config const geoDist: GeoDistrib  = GeoDistrib.GeoFixed;

  // Print out search parameters
  proc uts_showSearchParams() {
    writeln("UTS - Unbalanced Tree Search");

    writeln("Tree type: ", distrib);
    writeln("Tree shape parameters:");
    writeln("  root branching factor b_0 = ", B_0, ", root seed = ", SEED);
    
    if (distrib == NodeDistrib.Geometric || distrib == NodeDistrib.Hybrid) then
      writeln("  Geo. params: Max_Height = ", MAX_HEIGHT, ", shape function = ", geoDist);

    if (distrib == NodeDistrib.Binomial || distrib == NodeDistrib.Hybrid) then
      writeln("  Bin. params: nonLeafProb=", nonLeafProb, " nonLeafBF=", nonLeafBF);

    if (distrib == NodeDistrib.Hybrid) then
      writeln("  Hybrid params: shiftHeight=", shiftHeight);

    writeln("Random number generator: ", rng_getName());

    writeln();
  }

  proc main {
    var timer: Timer();
    uts_showSearchParams();
    timer.start();

    initPool();
    var root: TreeNode = new TreeNode(0);
    rng_init(root.hash[1], SEED);

    var result: uint = 0;
    result +=  root.generateAndDistributeInitTasks(): uint;
    result += compute();

    timer.stop();
    writeln("Number of tree nodes: " + result);
    writeln("Time: ", timer.elapsed(), " sec");
  }

