/*
** Main function, start of workers and main loop of workers 
*/

public class Uts {

  public static def main(args: Rail[String]) {
    val paras = new Paras(args);
    paras.showParas();
    val startTime:Long = System.nanoTime();

    val root: TreeNode = new TreeNode(0n, paras.seed);
    val tp = PlaceLocalHandle.make[InfosPerPlace](PlaceGroup.WORLD, ()=>new InfosPerPlace(paras));
    for (place in Place.places()) {
      at (place) {
        tp().init(tp);
      }
    }
    var nTotal: Long = root.initPool(tp);
    nTotal += compute(tp);
    val endTime = System.nanoTime();

    Console.OUT.println("Number of tree nodes: " + nTotal);
    Console.OUT.println("Time: " + ((endTime - startTime) / 1e9) + " sec");
  }

  // Run workers and count tree nodes
  static def compute(tp: PlaceLocalHandle[InfosPerPlace]): Long {
    finish {
      for (place in Place.places()) {
        async at (place) {
          finish {
            for (workerID in 0 .. (tp().paras.numWorkersPerPlace - 1)) {
              async {
                val myN = runWorker(workerID, tp); 
                atomic {
                  tp().nPlace += myN;
                }
              }
            }
          }
        }
      }
    }
    var nTotal: Long = 0;
    for (place in Place.places()) {
      nTotal += at (place) tp().nPlace;
    }
    return nTotal;
  }

  // Main loop of worker
  static def runWorker(workerID: Long, tp: PlaceLocalHandle[InfosPerPlace]): Long {
    val mySQ : SplitQueue = tp().sqsOfOnePlace(workerID);
    val paras = tp().paras;
    var myN: Long = 0;
    var node: TreeNode = mySQ.pop();
    while (node != null) {   // null = local split queue empty and stealing failed
      myN++;
      node.genChildrenHere(workerID, tp);
      node = mySQ.pop();
    }
    return myN;
  }

}
